@@ -152,35 +152,37 @@ sub new {
 }
 
 =item connect, listen, bind, getsockopt, setsockopt,
-send, recv, getpeername, getsockname
+send, recv, peername, sockname
 
-Do the same thing as the perl builtins (but return true on
-EINPROGRESS). Remember that these must be method calls.
+Do the same thing as the perl builtins or IO::Socket methods (but return
+true on EINPROGRESS). Remember that these must be method calls.
 
 =cut
 
-sub connect	{ connect tied(${$_[0]})->{fh}, $_[1] or $! == Errno::EINPROGRESS }
-sub bind	{ bind    tied(${$_[0]})->{fh}, $_[1] }
-sub listen	{ listen  tied(${$_[0]})->{fh}, $_[1] }
-sub getsockopt	{ getsockopt tied(${$_[0]})->{fh}, $_[1], $_[2] }
-sub setsockopt	{ setsockopt tied(${$_[0]})->{fh}, $_[1], $_[2], $_[3] }
-sub send	{ send tied(${$_[0]})->{fh}, $_[1], $_[2], @_ > 2 ? $_[3] : () }
-sub recv	{ recv tied(${$_[0]})->{fh}, $_[1], $_[2], @_ > 2 ? $_[3] : () }
-sub getsockname	{ getsockname tied(${$_[0]})->{fh} }
-sub getpeername	{ getpeername tied(${$_[0]})->{fh} }
+sub connect	{ connect tied(${$_[0]})->[0], $_[1] or $! == Errno::EINPROGRESS }
+sub bind	{ bind    tied(${$_[0]})->[0], $_[1] }
+sub listen	{ listen  tied(${$_[0]})->[0], $_[1] }
+sub getsockopt	{ getsockopt tied(${$_[0]})->[0], $_[1], $_[2] }
+sub setsockopt	{ setsockopt tied(${$_[0]})->[0], $_[1], $_[2], $_[3] }
+sub send	{ send tied(${$_[0]})->[0], $_[1], $_[2], @_ > 2 ? $_[3] : () }
+sub recv	{ recv tied(${$_[0]})->[0], $_[1], $_[2], @_ > 2 ? $_[3] : () }
+sub sockname	{ getsockname tied(${$_[0]})->[0] }
+sub peername	{ getpeername tied(${$_[0]})->[0] }
 
 =item ($peername, $fh) = $listen_fh->accept
 
 In scalar context, returns the newly accepted socket (or undef) and in
-list context return the ($peername, $fh) pair (or nothing).
+list context return the ($fh, $peername) pair (or nothing).
 
 =cut
 
 sub accept {
    my ($peername, $fh);
    while () {
-      $peername = accept $fh, tied(${$_[0]})->{fh}
-         and return ($peername, $fh = new_from_fh Coro::Socket $fh);
+      $peername = accept $fh, tied(${$_[0]})->[0]
+         and return wantarray 
+                    ? ((new_from_fh Coro::Socket $fh), $peername)
+                    : (new_from_fh Coro::Socket $fh);
 
       return unless $!{EAGAIN};
 
